@@ -50,6 +50,17 @@
 #' @param iter Number of samples per chain. .
 #' @param refresh Stan will print the progress of the sampler every \code{refresh} number of samples; set \code{refresh=0} to silence this.
 #' @param keep_all  If `keep_all = TRUE` then samples for all parameters in the Stan model will be kept; this is necessary if you want to do model comparison with Bayes factors and the `bridgesampling` package.
+#' @param slim If `slim = TRUE`, then the Stan model will not collect the most memory-intensive parameters (including n-length vectors of fitted values, log-likelihoods, and ME-modeled covariate values). This will disable many convenience functions that are otherwise available for fitted \code{geostan} models, such as the extraction of residuals, fitted values, and spatial trends, WAIC, and spatial diagnostics, and ME diagnostics; many quantities of interest, such as fitted values and spatial trends, can still be calculated manually using given parameter estimates. The "slim" option is designed for data-intensive routines, such as regression with raster data, Monte Carlo studies, and measurement error models. For more control over which parameters are kept or dropped, use the `drop` argument instead of `slim`.
+#' @param drop Provide a vector of character strings to specify the names of any parameters that you do not want MCMC samples for. Dropping parameters in this way can improve sampling speed and reduce memory usage. The following parameter vectors can potentially be dropped from GLM models:
+#' \describe{
+#' \item{fitted}{The N-length vector of fitted values}
+#' \item{log_lik}{The N-length vector of pointwise log-likelihoods, which is used to calculate WAIC.}
+#' \item{alpha_re}{Vector of 'random effects'/varying intercepts.}
+#' \item{x_true}{N-length vector of 'latent'/modeled covariate values created for measurement error (ME) models.}
+#' \item{phi}{The N-length vector of spatially-autocorrelated parameters (with the ICAR prior).}
+#' \item{theta}{The N-length vector of spatially unstructured parameters ('random effects'), for the BYM and BYM2 models.}
+#' }
+#' Using `drop = c('fitted', 'log_lik', 'alpha_re', 'x_true')` is equivalent to `slim = TRUE`. Note that if `slim = TRUE`, then `drop` will be ignored---so only use one or the other.
 #' @param pars Optional; specify any additional parameters you'd like stored from the Stan model.
 #' @param control A named list of parameters to control the sampler's behavior. See \code{\link[rstan]{stan}} for details. 
 #' @param ... Other arguments passed to \link[rstan]{sampling}. For multi-core processing, you can use \code{cores = parallel::detectCores()}, or run \code{options(mc.cores = parallel::detectCores())} first.
@@ -59,45 +70,49 @@
 #'
 #' The exact specification depends on the `type` argument. 
 #'
-#' ### 'icar'
+#' ### ICAR
 #'
 #' For Poisson models for count data, y, the basic model specification (`type = "icar"`) is:
-#' \deqn{
-#' y ~ Poisson(e^{O + \mu + \phi}) \\
-#' \phi \sim ICAR(\tau_s) \\
-#' \tau_s \sim Gauss(0, 1)
-#' }
+#' 
+#' \deqn{y ~ Poisson(e^{O + \mu + \phi}) }
+#' \deqn{\phi \sim ICAR(\tau_s) }
+#' \deqn{\tau_s \sim Gauss(0, 1)}
+#' 
 #'  where \eqn{\mu} contains an intercept and potentially covariates. The spatial trend \eqn{phi} has a mean of zero and a single scale parameter \eqn{\tau_s} (which user's will see printed as the parameter named `spatial_scale`).
 #' 
-#' The ICAR prior model is a CAR model that has a spatial autocorrelation parameter \eqn{\rho} equal to 1 (see \link[geostan]{stan_car}). Thus the ICAR prior places high probability on a very smooth spatially (or temporally) varying mean. This is rarely sufficient to model the amount of variation present in social and health data.
+#' The ICAR prior model is a CAR model that has a spatial autocorrelation parameter \eqn{\rho} equal to 1 (see \link[geostan]{stan_car}). Thus the ICAR prior places high probability on a very smooth spatially (or temporally) varying mean. This is rarely sufficient to model the amount of variation present in social and health data. For this reason, the BYM model is typically employed.
 #'
-#' ### 'bym'
+#' ### BYM
 #'
 #' Often, an observational-level random effect term, `theta`, is added to capture (heterogeneous or unstructured) deviations from \eqn{\mu + \phi}. The combined term is referred to as a convolution term:
+#' 
 #' \eqn{
 #'  convolution = \phi + \theta.
 #' }
-#' This is known as the BYM model (Besag et al. 1991), and can be specified using `type = "bym"`:
-#' \eqn{
-#' y \sim Poisson(e^{O + \mu + \phi + \theta}) \\
-#' \phi \sim ICAR(\tau_s) \\
-#' \theta \sim Gaussian(0, \tau_{ns})
-#' \tau_s \sim Gaussian(0, 1)
-#' \tau_{ns} \sim Gaussian(0, 1)
-#' }
-#'
-#' ### 'bym2'
 #' 
-#' Riebler et al. (2016) introduce a variation on the BYM model (`type = "bym2"`). This specification combines \eqn{\phi} and \eqn{\theta} using a mixing parameter \eqn{\rho} that controls the proportion of the variation that is attributable to the spatially autocorrelated term \eqn{\phi} rather than the spatially unstructured term \eqn{\theta}. The terms share a single scale parameter:
-#' \deqn{
-#' convolution = [sqrt(\rho * scale_factor) * \tilde{\phi} + sqrt(1 - \rho)  \tilde{\theta}] * \tau_s \\
-#' \tilde{\phi} \sim Gaussian(0, 1) \\
-#' \tilde{\theta} \sim Gaussian(0, 1) \\
-#' \tau_s \sim Gaussian(0, 1)
-#' }
-#' The terms \eqn{\tilde{\phi}}, \eqn{\tilde{\theta}} are standard normal deviates, \eqn{\rho} is restricted to values between zero and one, and the 'scale_factor' is a constant term provided by the user. By default, the 'scale_factor' is equal to one, so that it does nothing. Riebler et al. (2016) argue that the interpretation or meaning of the scale of the ICAR model depends on the graph structure of the connectivity matrix \eqn{C}. This implies that the same prior distribution assigned to \eqn{\tau_s} will differ in its implications if \eqn{C} is changed; in other words, the priors are not transportable across models, and models that use the same nominal prior actually have different priors assigned to \eqn{\tau_s}.
+#' This is known as the BYM model (Besag et al. 1991), and can be specified using `type = "bym"`:
 #'
-#' Borrowing `R` code from Morris (2017) and following Freni-Sterrantino et al. (2018), the following `R` code can be used to create the 'scale_factor' for the BYM2 model (note, this requires the INLA R package), given a spatial adjacency matrix, \eqn{C}:
+#' \eqn{y \sim Poisson(e^{O + \mu + \phi + \theta}) }
+#' \deqn{ \phi \sim ICAR(\tau_s) }
+#' \deqn{ \theta \sim Gaussian(0, \tau_{ns}) }
+#' \deqn{ \tau_s \sim Gaussian(0, 1) }
+#' \deqn{ \tau_{ns} \sim Gaussian(0, 1) }
+#'
+#' The model is named after Besag, York, and Mollié (1991).
+#' 
+#' ### BYM2
+#' 
+#' Riebler et al. (2016) introduce a variation on the BYM model (`type = "bym2"`). This specification combines \eqn{\phi} and \eqn{\theta} using a mixing parameter \eqn{\rho} that controls the proportion of the variation that is attributable to the spatially autocorrelated term \eqn{\phi} rather than the spatially unstructured term \eqn{\theta}. The terms share a single scale parameter \eqn{\tau}:
+#' 
+#' \deqn{convolution = [sqrt(\rho * S) * \tilde{\phi} + sqrt(1 - \rho)  \tilde{\theta}] * \tau
+#' }
+#' \deqn{ \tilde{\phi} \sim Gaussian(0, 1) }
+#' \deqn{ \tilde{\theta} \sim Gaussian(0, 1) }
+#' \deqn{ \tau \sim Gaussian(0, 1) }
+#' 
+#' The terms \eqn{\tilde{\phi}}, \eqn{\tilde{\theta}} are standard normal deviates, \eqn{\rho} is restricted to values between zero and one, and \eqn{S} is the 'scale_factor' (a constant term provided by the user). By default, the 'scale_factor' is equal to one, so that it does nothing. Riebler et al. (2016) argue that the interpretation or meaning of the scale of the ICAR model depends on the graph structure of the connectivity matrix \eqn{C}. This implies that the same prior distribution assigned to \eqn{\tau_s} will differ in its implications if \eqn{C} is changed; in other words, the priors are not transportable across models, and models that use the same nominal prior actually have different priors assigned to \eqn{\tau_s}.
+#'
+#' Borrowing `R` code from Morris (2017) and following Freni-Sterrantino et al. (2018), the following `R` code can be used to create the 'scale_factor' \eqn{S} for the BYM2 model (note, this requires the INLA R package), given a spatial adjacency matrix, \eqn{C}:
 #' ```
 #' ## create a list of data for stan_icar
 #' icar.data <- geostan::prep_icar_data(C)
@@ -114,7 +129,7 @@
 #'   scale_factor[j] <- scale_c(Cg) 
 #' }
 #' ```
-#' This code adjusts for 'islands' or areas with zero neighbors, and it also handles disconnected graph structures (see Donegan 2021). Following Freni-Sterrantino (2018), disconnected components of the graph structure are given their own intercept term; however, this value is added to \eqn{\phi} automatically inside the Stan model. Therefore, the user never needs to make any adjustments for this term. (If you want to avoid complications from a disconnected graph structure, see \code{\link[geostan]{stan_car}}).
+#' This code adjusts for 'islands' or areas with zero neighbors, and it also handles disconnected graph structures (see Donegan and Morris 2021). Following Freni-Sterrantino (2018), disconnected components of the graph structure are given their own intercept term; however, this value is added to \eqn{\phi} automatically inside the Stan model. Therefore, the user never needs to make any adjustments for this term. (To avoid complications from using a disconnected graph structure, you can apply a proper CAR model instead of the ICAR: \code{\link[geostan]{stan_car}}).
 #' 
 #' Note, the code above requires the `scale_c` function; it has package dependencies that are not included in `geostan`. To use `scale_c`, you have to load the following `R` function:
 #' ```
@@ -145,78 +160,9 @@
 #'  return(scaling_factor) 
 #'}
 #' ```
+#' ## Additional functionality
 #'
-#' ### Spatially lagged covariates (SLX)
-#' 
-#' The `slx` argument is a convenience function for including SLX terms. For example, 
-#' \deqn{
-#'  y = W X \gamma + X \beta + \epsilon
-#' }
-#' where \eqn{W} is a row-standardized spatial weights matrix (see \link[geostan]{shape2mat}), \eqn{WX} is the mean neighboring value of \eqn{X}, and \eqn{\gamma} is a coefficient vector. This specifies a regression with spatially lagged covariates. SLX terms can specified by providing a formula to the \code{slx} argument:
-#' ```
-#' stan_glm(y ~ x1 + x2, slx = ~ x1 + x2, \...),
-#' ```
-#' which is a shortcut for
-#' ```
-#' stan_glm(y ~ I(W \%*\% x1) + I(W \%*\% x2) + x1 + x2, \...)
-#' ```
-#' SLX terms will always be *prepended* to the design matrix, as above, which is important to know when setting prior distributions for regression coefficients.
-#'
-#' For measurement error (ME) models, the SLX argument is the only way to include spatially lagged covariates since the SLX term needs to be re-calculated on each iteration of the MCMC algorithm.
-#' 
-#' ### Measurement error (ME) models
-#' 
-#' The ME models are designed for surveys with spatial sampling designs, such as the American Community Survey (ACS) estimates. Given estimates \eqn{x}, their standard errors \eqn{s}, and the target quantity of interest (i.e., the unknown true value) \eqn{z}, the ME models have one of the the following two specifications, depending on the user input. If a spatial CAR model is specified, then:
-#' \deqn{
-#'  x \sim Gauss(z, s^2) \\
-#'  z \sim Gauss(\mu_z, \Sigma_z) \\
-#' \Sigma_z = (I - \rho C)^{-1} M \\
-#'  \mu_z \sim Gauss(0, 100) \\
-#'  \tau_z \sim Student(10, 0, 40), \tau > 0 \\
-#'  \rho_z \sim uniform(l, u)
-#'  }
-#' where \eqn{\Sigma} specifies a spatial conditional autoregressive model with scale parameter \eqn{\tau} (on the diagonal of \eqn{M}), and \eqn{l}, \eqn{u} are the lower and upper bounds that \eqn{\rho} is permitted to take (which is determined by the extreme eigenvalues of the spatial connectivity matrix \eqn{C}).
-#' 
-#' For non-spatial ME models, the following is used instead:
-#' \deqn{
-#' x \sim Gauss(z, s^2) \\
-#' z \sim student(\nu_z, \mu_z, \sigma_z) \\
-#' \nu_z \sim gamma(3, 0.2) \\
-#' \mu_z \sim Gauss(0, 100) \\
-#' \sigma_z \sim student(10, 0, 40).
-#' }
-#' 
-#' For strongly skewed variables, such as census tract poverty rates, it can be advantageous to apply a logit transformation to \eqn{z} before applying the CAR or Student-t prior model. When the `logit` argument is used, the model becomes:
-#' \deqn{
-#' x \sim Gauss(z, s^2) \\
-#' logit(z) \sim Gauss(\mu_z, \Sigma_z) 
-#' ...
-#' }
-#' and similarly for the Student t model:
-#' \deqn{
-#' x \sim Gauss(z, s^2) \\
-#' logit(z) \sim student(\nu_z, \mu_z, \sigma_z) \\
-#' ...
-#' }
-#'
-#' ### Censored counts
-#'
-#' Vital statistics systems and disease surveillance programs typically suppress case counts when they are smaller than a specific threshold value. In such cases, the observation of a censored count is not the same as a missing value; instead, you are informed that the value is an integer somewhere between zero and the threshold value. For Poisson models (`family = poisson())`), you can use the `censor_point` argument to encode this information into your model. 
-#'
-#' Internally, `geostan` will keep the index values of each censored observation, and the index value of each of the fully observed outcome values. For all observed counts, the likelihood statement will be:
-#' \deqn{
-#' p(y_i | data, model) = poisson(y_i | \mu_i), 
-#' }
-#' as usual, where \eqn{\mu_i} may include whatever spatial terms are present in the model.
-#'
-#' For each censored count, the likelihood statement will equal the cumulative Poisson distribution function for values zero through the censor point:
-#' \deqn{
-#' p(y_i | data, model) = \sum_{m=0}^{M} Poisson( m | \mu_i),
-#' }
-#' where \eqn{M} is the censor point and \eqn{\mu_i} again is the fitted value for the \eqn{i^{th}} observation.
-#' 
-#' For example, the US Centers for Disease Control and Prevention's CDC WONDER database censors all death counts between 0 and 9. To model CDC WONDER mortality data, you could provide `censor_point = 9` and then the likelihood statement for censored counts would equal the summation of the Poisson probability mass function over each integer ranging from zero through 9 (inclusive), conditional on the fitted values (i.e., all model parameters). See Donegan (2021) for additional discussion, references, and Stan code.
-#'
+#' The CAR models can also incorporate spatially-lagged covariates, measurement/sampling error in covariates (particularly when using small area survey estimates as covariates), and censored outcomes (such as arise when a disease surveillance system suppresses data for privacy reasons). For details on these options, please see the Details section in the documentation for \link[geostan]{stan_glm}.
 #' 
 #' @return An object of class class \code{geostan_fit} (a list) containing: 
 #' \describe{
@@ -243,15 +189,15 @@
 #'
 #' Besag, J. (1974). Spatial interaction and the statistical analysis of lattice systems. *Journal of the Royal Statistical Society: Series B (Methodological)*, 36(2), 192-225.
 #'
-#' Besag, J., York, J., & Mollié, A. (1991). Bayesian image restoration, with two applications in spatial statistics. *Annals of the Institute of Statistical Mathematics*, 43(1), 1-20.
+#' Besag, J., York, J., and Mollié, A. (1991). Bayesian image restoration, with two applications in spatial statistics. *Annals of the Institute of Statistical Mathematics*, 43(1), 1-20.
 #'
-#' Donegan, Connor. 2021. Flexible functions for ICAR, BYM, and BYM2 models in Stan. Code repository. <https://github.com/ConnorDonegan/Stan-IAR>
+#' Donegan, Connor and Morris, Mitzi (2021). Flexible functions for ICAR, BYM, and BYM2 models in Stan. Code repository. <https://github.com/ConnorDonegan/Stan-IAR>
+#'
+#' Donegan, Connor (2021b). Building spatial conditional autoregressive (CAR) models in the Stan programming language. *OSF Preprints*. \doi{10.31219/osf.io/3ey65}.
 #'
 #' Donegan, Connor and Chun, Yongwan and Griffith, Daniel A. (2021). Modeling community health with areal data: Bayesian inference with survey standard errors and spatial structure. *Int. J. Env. Res. and Public Health* 18 (13): 6856. DOI: 10.3390/ijerph18136856 Data and code: \url{https://github.com/ConnorDonegan/survey-HBM}.
-#'
-#' Donegan, Connor (2021). Spatial conditional autoregressive models in Stan. *OSF Preprints*. \doi{10.31219/osf.io/3ey65}.
 #' 
-#' Freni-Sterrantino, Anna, Massimo Ventrucci, and Håvard Rue. 2018. A Note on Intrinsic Conditional Autoregressive Models for Disconnected Graphs. *Spatial and Spatio-Temporal Epidemiology*, 26: 25–34.
+#' Freni-Sterrantino, Anna, Massimo Ventrucci, and Håvard Rue (2018). A Note on Intrinsic Conditional Autoregressive Models for Disconnected Graphs. *Spatial and Spatio-Temporal Epidemiology*, 26: 25–34.
 #' 
 #' Morris, M., Wheeler-Martin, K., Simpson, D., Mooney, S. J., Gelman, A., & DiMaggio, C. (2019). Bayesian hierarchical spatial models: Implementing the Besag York Mollié model in stan. *Spatial and spatio-temporal epidemiology*, 31, 100301.
 #'
@@ -321,6 +267,8 @@ stan_icar <- function(formula,
                       prior_only = FALSE,
                       chains = 4, iter = 2e3, refresh = 500,
                       keep_all = FALSE,
+                      slim = FALSE,
+                      drop = NULL,
                       pars = NULL,
                       control = NULL,
                       ...) {
@@ -442,9 +390,9 @@ stan_icar <- function(formula,
       standata$trials <- y[,1] + y[,2]
     }
     ## PARAMETERS TO KEEP with ICAR [START] -------------        
-    pars <- c(pars, 'intercept', 'log_lik', 'fitted', 'phi', 'spatial_scale')
-    if (type == "bym2") pars <- c(pars, "theta", "rho")
-    if (type == "bym") pars <- c(pars, "theta", "theta_scale")
+    pars <- c(pars, 'intercept', 'spatial_scale', 'fitted', 'phi', 'log_lik')
+    if (type == "bym2") pars <- c(pars, 'theta', 'rho')
+    if (type == "bym") pars <- c(pars, 'theta', 'theta_scale')
     if (standata$m) pars <- c(pars, "alpha_phi")
     if (!intercept_only) pars <- c(pars, 'beta')
     if (dwx) pars <- c(pars, 'gamma')
@@ -457,6 +405,8 @@ stan_icar <- function(formula,
             pars <- c(pars, "nu_x_true")
         }
     }
+    if (slim == TRUE) drop <- c('fitted', 'log_lik', 'alpha_re', 'x_true', 'phi', 'theta')
+    pars <- drop_params(pars = pars, drop_list = drop)
     priors_made_slim <- priors_made[which(names(priors_made) %in% pars)]
     if (me.list$has_me) priors_made_slim$ME_model <- ME$prior        
     print_priors(prior, priors_made_slim)    
@@ -490,8 +440,10 @@ stan_icar <- function(formula,
     out$edges <- edges(C)       
     out$spatial <- data.frame(par = "phi", method = toupper(type))
     ## ICAR OUTPUT [STOP] --------
-    R <- resid(out, summary = FALSE)
-    out$diagnostic["Residual_MC"] <- mean( apply(R, 1, mc, w = C, warn = FALSE, na.rm = TRUE) )    
+    if (!missing(C) && any(pars == 'fitted')) {
+        R <- resid(out, summary = FALSE)
+        out$diagnostic["Residual_MC"] <- mean( apply(R, 1, mc, w = C, warn = FALSE, na.rm = TRUE) )
+    }
     return (out)
 }
 
